package com.cecf.observe;

import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.configuration.RestartStrategyOptions;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.table.api.EnvironmentSettings;
import org.apache.flink.table.api.Table;
import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
import org.apache.flink.types.Row;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.ivi.opensource.flinkclickhousesink.model.ClickHouseClusterSettings;
import ru.ivi.opensource.flinkclickhousesink.model.ClickHouseSinkConst;

import java.net.MalformedURLException;
import java.net.URL;
import java.time.Duration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

public class TraceStream {

    private static String getProperty(String key, String de) {
        String v = System.getProperty(key);
        if (v == null) {
            return de;
        }
        return v;
    }

    public static final Logger logger = LoggerFactory.getLogger(TraceStream.class);

    static KafkaSource<String> buildKafkaSource() {
        String brokers = getProperty("KAFKA_BROKERS", "kafka-headless.observe.svc.cluster.local:9092");
        return KafkaSource.<String>builder()
                .setBootstrapServers(brokers)
                .setTopics(getProperty("KAFKA_TOPIC", "otel"))
                .setGroupId(getProperty("KAFKA_GROUP", "ob-agent-stream"))
                .setStartingOffsets(OffsetsInitializer.latest())
                .setValueOnlyDeserializer(new SimpleStringSchema())
                .build();
    }

    public static int parsePortFromUrl(String urlString) {
        try {
            URL url = new URL(urlString);
            String portString = url.getAuthority().split(":")[1];
            return Integer.parseInt(portString);
        } catch (MalformedURLException | ArrayIndexOutOfBoundsException e) {
            if ("http".equalsIgnoreCase(urlString.split(":")[0])) {
                return 80;
            } else if ("https".equalsIgnoreCase(urlString.split(":")[0])) {
                return 443;
            }
            return -1;
        }
    }

    public static void main(String[] args) throws Exception {
        Configuration envConfig = new Configuration();
        envConfig.set(RestartStrategyOptions.RESTART_STRATEGY, "fixed-delay");
        envConfig.set(RestartStrategyOptions.RESTART_STRATEGY_FIXED_DELAY_ATTEMPTS, Integer.MAX_VALUE); // number of restart attempts
        envConfig.set(RestartStrategyOptions.RESTART_STRATEGY_FIXED_DELAY_DELAY, Duration.ofSeconds(10)); // delay
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment(envConfig);
        Map<String, String> globalParameters = new HashMap<>();

        String chHosts = getProperty("CH_HOST", "clickhouse.observe.svc.cluster.local");
        String chPort = getProperty("CH_PORT", "8123");
        String chUser = getProperty("CH_USER", "default");
        String chPwd = getProperty("CH_PWD", "cecf@cestong.com");
        globalParameters.put(ClickHouseClusterSettings.CLICKHOUSE_HOSTS, chHosts);
        globalParameters.put(ClickHouseClusterSettings.CLICKHOUSE_USER, chUser);
        globalParameters.put(ClickHouseClusterSettings.CLICKHOUSE_PASSWORD, chPwd);

        globalParameters.put(ClickHouseSinkConst.TIMEOUT_SEC, getProperty("CH_TIMEOUT", "10"));
        globalParameters.put(ClickHouseSinkConst.FAILED_RECORDS_PATH, "./failed_record");
        globalParameters.put(ClickHouseSinkConst.NUM_WRITERS, getProperty("CH_WRITER_NUM", "1"));
        globalParameters.put(ClickHouseSinkConst.NUM_RETRIES, getProperty("CH_RETRY_NUM", "3"));
        globalParameters.put(ClickHouseSinkConst.QUEUE_MAX_CAPACITY, getProperty("CH_WRITE_QUEUE_SIZE", "10"));
        globalParameters.put(ClickHouseSinkConst.IGNORING_CLICKHOUSE_SENDING_EXCEPTION_ENABLED, "true");

        String excludeService = getProperty("EXCLUDE_SERVICE", "");
        String[] excludeServiceList = excludeService.split(",");

        ParameterTool parameters = ParameterTool.fromMap(globalParameters);
        env.getConfig().setGlobalJobParameters(parameters);
//        KafkaSource<String> source = buildKafkaSource();

        Properties props = new Properties();
        props.put(ClickHouseSinkConst.TARGET_TABLE_NAME, getProperty("CH_TABLE_NAME", "otel.traces_local"));
        props.put(ClickHouseSinkConst.MAX_BUFFER_SIZE, getProperty("CH_WRITE_BUF_SIZE", "1000000000"));

        SpringBootSpanCSVConverter converter = new SpringBootSpanCSVConverter();


        String mysqlDSN = getProperty("MYSQL_DSN", "jdbc:mysql://m1.cestong.com.cn:31511/observe");
        String mysqlUserName = getProperty("MYSQL_USER", "root");
        String mysqlPwd = getProperty("MYSQL_PWD", "pg3mfWRtYonekZWB");

        EnvironmentSettings settings = EnvironmentSettings
                .newInstance()
                .inStreamingMode()
                .build();


        StreamTableEnvironment tEnv = StreamTableEnvironment.create(env);
        String queryLogTblName = "t_query_log";
        String sql = String.format(" create table %s (`hostname` STRING,\n" +
                                   " `type` STRING,\n" + " `event_date` DATE,\n" + " `event_time` TIMESTAMP(3),\n" + " `query_start_time` TIMESTAMP,\n" +
                                   " `query_duration_ms` DECIMAL,\n" + " `read_rows` DECIMAL,\n" + " `read_bytes` DECIMAL,\n" + " `written_rows` DECIMAL,\n" +
                                   " `written_bytes` DECIMAL,\n" + " `result_rows` DECIMAL,\n" + " `result_bytes` DECIMAL,\n" + " `memory_usage` DECIMAL,\n" +
                                   " `current_database` STRING,\n" + " `query` STRING,\n" + " `formatted_query` STRING,\n" + " `normalized_query_hash` DECIMAL(20, 0),\n" +
                                   " `query_kind` STRING,\n" + " `databases` ARRAY<STRING>,\n" + " `tables` ARRAY<STRING>,\n" + " `columns` ARRAY<STRING>,\n" +
                                   " `partitions` ARRAY<STRING>,\n" + " `exception_code` INTEGER,\n" + " `exception` STRING,\n" + " `stack_trace` STRING,\n" +
                                   " `is_initial_query` SMALLINT,\n" + " `user` STRING,\n" + " `query_id` STRING,\n" + " `port` INTEGER,\n" + " `client_name` STRING,\n" +
                                   " `http_user_agent` STRING, \n" +
                                   "  WATERMARK FOR event_time AS event_time - INTERVAL '5' SECOND" +
                                   " ) with (\n" +
                                   "    'connector' = 'clickhouse',\n" +
                                   "    'url' = 'clickhouse://%s:%s',\n" +
                                   "    'database-name' = 'system',\n" +
                                   "    'table-name' = 'query_log',\n" +
                                   "    'username' = '%s',\n" +
                                   "    'password' = '%s'\n" +
                                   ")\n", queryLogTblName, chHosts, chPort, chUser, chPwd);
        tEnv.executeSql(sql);
        Table inputTable = tEnv.sqlQuery(String.format("select * from %s", queryLogTblName));
        DataStream<QueryLogRow> queryLogRowDataStream = tEnv.toDataStream(inputTable).map(new MapRow2QueryLog()).name("TableRow2QueryLogStruct");

        queryLogRowDataStream
                .filter((FilterFunction<QueryLogRow>) queryLogRow -> queryLogRow.exceptionCode > 0)
                .windowAll(TumblingEventTimeWindows.of(Duration.ofMinutes(5)))
                .aggregate(new ErrorCountAggregate(), new TimeWindowAdder<>())
                .name("CountByExceptionCode")
                .print();

//        env.fromSource(source, WatermarkStrategy.noWatermarks(), "KafkaSource")
//                .map(new SpringBootSpanExtractMapFunc())
//                .name("InfoExtract")
//                .flatMap(new SpringBootFlatMapFunc())
//                .name("ResourceSpansFlatToSpan")
//                .filter(new ServiceNameFilter(excludeServiceList))
//                .name("FilterByServiceName")
//                .map(new FillAppNameMapFunc(mysqlDSN, mysqlUserName, mysqlPwd))
//                .name("AttachAppName")
//                .addSink(new ClickHouseSink<>(props, converter))
//                .name("ClickhouseSink");

        logger.info("配置完成，运行flink");
        env.execute(getProperty("FLINK_JOB_NAME", "Ob stream operator"));
    }


}
